"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathResolver = void 0;
const SchemaResolver_1 = require("./SchemaResolver");
const lodash_es_1 = require("lodash-es");
const constants_1 = require("../constants");
const specifications_1 = require("../utils/specifications");
class PathResolver {
    constructor(paths) {
        this.paths = paths;
        this.resolvedPaths = [];
        this.extraDefinitions = {};
        this.contentType = {};
        this.resolve = () => {
            this.resolvedPaths = (0, lodash_es_1.reduce)(this.paths, (results, path, pathName) => [...results, ...this.resolvePath(path, pathName)], []);
            return this;
        };
        this.getRequestURL = (pathName) => {
            return pathName
                .split(constants_1.SLASH)
                .map((p) => (this.isPathParam(p) ? `$${p}` : p))
                .join(constants_1.SLASH);
        };
        this.isPathParam = (str) => str.startsWith("{");
        this.resolveOperation = (operation) => {
            const pickParamsByType = this.pickParams(operation.parameters);
            const headerParams = pickParamsByType("header");
            const params = {
                pathParams: pickParamsByType("path"),
                queryParams: pickParamsByType("query"),
                bodyParams: pickParamsByType("body"),
                formDataParams: pickParamsByType("cookie"),
            };
            return Object.assign(Object.assign({ operationId: operation.operationId, TResp: this.getResponseTypes(operation.responses), TReq: this.getRequestTypes(params, operation.operationId, (0, lodash_es_1.get)(operation, "requestBody")), THeader: this.getPathParamsTypes(headerParams) }, this.getParamsNames(params)), this.getRequestBodyName((0, lodash_es_1.get)(operation, "requestBody"), operation.operationId));
        };
        this.getParamsNames = (params) => {
            const getNames = (list) => ((0, lodash_es_1.isEmpty)(list) ? [] : (0, lodash_es_1.map)(list, (item) => item.name));
            return {
                pathParams: getNames(params.pathParams),
                queryParams: getNames(params.queryParams),
                bodyParams: getNames(params.bodyParams),
                formDataParams: getNames(params.formDataParams),
            };
        };
        this.getRequestTypes = (params, operationId, requestBody) => (Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, this.getPathParamsTypes(params.pathParams)), this.getBodyAndQueryParamsTypes(params.bodyParams)), this.getBodyAndQueryParamsTypes(params.queryParams)), this.getFormDataParamsTypes(params.formDataParams)), this.getRequestBodyTypes(operationId, requestBody)));
        this.getPathParamsTypes = (pathParams) => pathParams.reduce((results, param) => {
            const schema = (0, lodash_es_1.get)(param, "schema");
            if ((0, specifications_1.isSchema)(schema)) {
                return Object.assign(Object.assign({}, results), { [`${param.name}${param.required ? "" : "?"}`]: schema.type === "integer" ? "number" : schema.type });
            }
            return Object.assign({}, results);
        }, {});
        this.getBodyAndQueryParamsTypes = (bodyParams) => bodyParams.reduce((results, param) => (Object.assign(Object.assign({}, results), { [`${param.name}${param.required ? "" : "?"}`]: SchemaResolver_1.SchemaResolver.of({
                results: this.extraDefinitions,
                schema: param.schema,
                key: param.name,
                parentKey: param.name,
            })
                .resolve()
                .getSchemaType() })), {});
        this.getFormDataParamsTypes = (formDataParams) => {
            return formDataParams.reduce((results, param) => {
                if (param.schema) {
                    return Object.assign(Object.assign({}, results), { [`${param.name}${param.required ? "" : "?"}`]: SchemaResolver_1.SchemaResolver.of({
                            results: this.extraDefinitions,
                            schema: param.schema,
                            key: param.name,
                            parentKey: param.name,
                        })
                            .resolve()
                            .getSchemaType() });
                }
                return Object.assign(Object.assign({}, results), { [`${param.name}${param.required ? "" : "?"}`]: param.type === "file" ? "File" : param.type });
            }, {});
        };
        // TODO: handle Response or Reference
        this.getResponseTypes = (responses) => SchemaResolver_1.SchemaResolver.of({
            results: this.extraDefinitions,
            // TODO: handle other content type here
            schema: (0, lodash_es_1.get)(responses, "200.content.application/json.schema") ||
                (0, lodash_es_1.get)(responses, "200.content.*/*.schema") ||
                (0, lodash_es_1.get)(responses, "201.content.application/json.schema") ||
                (0, lodash_es_1.get)(responses, "201.content.*/*.schema"),
        })
            .resolve()
            .getSchemaType();
        // TODO: when parameters has enum
        this.pickParams = (parameters) => (type) => (0, lodash_es_1.filter)(parameters, (param) => param.in === type);
    }
    static of(paths) {
        return new PathResolver(paths);
    }
    resolvePath(path, pathName) {
        const operations = (0, lodash_es_1.pick)(path, constants_1.HTTP_METHODS);
        return Object.keys(operations).map((httpMethod) => (Object.assign({ url: this.getRequestURL(pathName), method: httpMethod }, this.resolveOperation(operations[httpMethod]))));
    }
    getContentType(operationId, key) {
        // in openAPI spec, the key of content in requestBody field is content type
        (0, lodash_es_1.assign)(this.contentType, { [operationId]: key });
    }
    getRequestBodyTypes(operationId, requestBody) {
        if ((0, specifications_1.isRequestBody)(requestBody)) {
            return (0, lodash_es_1.reduce)((0, lodash_es_1.get)(requestBody, "content"), (results, content, key) => {
                this.getContentType(operationId, key);
                return Object.assign(Object.assign({}, results), { [`${operationId}Request`]: SchemaResolver_1.SchemaResolver.of({
                        results: this.extraDefinitions,
                        schema: content.schema,
                        key: `${operationId}Request`,
                        parentKey: `${operationId}Request`,
                    })
                        .resolve()
                        .getSchemaType() });
            }, {});
        }
        return {
            [`${operationId}Request`]: SchemaResolver_1.SchemaResolver.of({
                results: this.extraDefinitions,
                schema: requestBody,
                key: `${operationId}Request`,
                parentKey: `${operationId}Request`,
            })
                .resolve()
                .getSchemaType(),
        };
    }
    getRequestBodyName(requestBody, operationId) {
        if (requestBody) {
            return {
                requestBody: `${operationId}Request`,
            };
        }
    }
}
exports.PathResolver = PathResolver;
