'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var lodashEs = require('lodash-es');
var tslib = require('tslib');

var isArray = function (data) {
  return Object.prototype.toString.call(data) === "[object Array]";
};
var isObject = function (data) {
  return Object.prototype.toString.call(data) === "[object Object]";
};
var isNumber = function (n) {
  return !isNaN( /*#__PURE__*/parseFloat(n)) && isFinite(n);
};
var isSchema = function (schema) {
  return !lodashEs.has(schema, "$ref");
};
var isRequestBody = function (requestBody) {
  return !lodashEs.has(requestBody, "$ref");
};

var HTTP_METHODS = ["get", "post", "put", "delete", "patch", "options", "head"];
var SLASH = "/";
var ENUM_SUFFIX = "#EnumTypeSuffix";

var toCapitalCase = function (str) {
  if (!str) {
    return "";
  }

  var camelStr = /*#__PURE__*/lodashEs.camelCase(str);
  return "" + camelStr.charAt(0).toUpperCase() + camelStr.slice(1);
};

var addPrefix = function (prefix) {
  return function (str) {
    if (str === void 0) {
      str = "";
    }

    return "" + prefix + str;
  };
};

var addPrefixForInterface = /*#__PURE__*/addPrefix("I");
var arrayToObject = function (arr) {
  if (arr === void 0) {
    arr = [];
  }

  var obj = {};
  lodashEs.forEach(arr, function (item) {
    obj[item] = item;
  });
  return obj;
};
var toTypes = function (definitions) {
  if (lodashEs.isEmpty(definitions)) {
    return;
  }

  var fieldDefinitionList = /*#__PURE__*/lodashEs.map(definitions, function (value, key) {
    if (isObject(value) && Object.keys(value).length === 1 && !lodashEs.isEmpty( /*#__PURE__*/lodashEs.pickBy(value, function (type) {
      return type === "FormData";
    }))) {
      return convertKeyToCamelCaseAndAddQuote(key) + ": FormData";
    }

    return isObject(value) ? convertKeyToCamelCaseAndAddQuote(key) + ": " + JSON.stringify(value).replace(/"/g, "") + ";" : convertKeyToCamelCaseAndAddQuote(key) + ": " + lodashEs.replace(value, ENUM_SUFFIX, "") + ";";
  });
  return definitions && "{\n        " + fieldDefinitionList.sort().join("\n") + "\n      }";
};

var convertKeyToCamelCaseAndAddQuote = function (key) {
  var isOptional = lodashEs.indexOf(key, "?") > -1;
  return "'" + lodashEs.camelCase( /*#__PURE__*/toCapitalCase( /*#__PURE__*/lodashEs.trimEnd(key, "?"))) + "'" + (isOptional ? "?" : "");
};

var SchemaResolver = function () {
  function SchemaResolver(inputs) {
    var _this = this;

    this.inputs = inputs;
    this.schemaType = {};

    this.getSchemaType = function () {
      return _this.schemaType;
    };

    this.resolve = function (type) {
      var _a = _this.inputs,
          _b = _a.schema,
          schema = _b === void 0 ? {} : _b,
          results = _a.results,
          parentKey = _a.parentKey,
          key = _a.key;

      var advancedType = /*#__PURE__*/_this.resolveRef(schema.$ref, type || schema.type);

      if (schema.$ref) {
        _this.schemaType = advancedType;

        _this.resolveNullable();

        return _this;
      }

      if (schema.items) {
        _this.schemaType = /*#__PURE__*/_this.resolveItems(schema.items, schema.type, key, parentKey);

        _this.resolveNullable();

        return _this;
      }

      if (schema.enum) {
        var enumKey = /*#__PURE__*/_this.getEnumName(key, parentKey); // TODO: should not do this operation here
        // Implicit operation!: Assign enum array definition to results


        results[enumKey] = schema.enum;
        _this.schemaType = enumKey;

        _this.resolveNullable();

        return _this;
      }

      if (schema.type === "object") {
        if (schema.properties) {
          _this.schemaType = /*#__PURE__*/_this.resolveProperties(schema.properties, schema.required, parentKey);

          _this.resolveNullable();

          return _this;
        }

        if (schema.title) {
          _this.schemaType = schema.type;

          _this.resolveNullable();

          return _this;
        }

        _this.schemaType = "{[key:string]:any}";

        _this.resolveNullable();

        return _this;
      }

      if (schema.type === "string" && schema.format === "binary") {
        _this.schemaType = "FormData";

        _this.resolveNullable();

        return _this;
      }

      _this.schemaType = /*#__PURE__*/_this.getBasicType(schema.type, advancedType);

      _this.resolveNullable();

      return _this;
    };

    this.resolveNullable = function () {
      var _a;

      if ((_a = _this.inputs.schema) === null || _a === void 0 ? void 0 : _a.nullable) {
        _this.schemaType = isObject(_this.schemaType) ? JSON.stringify(_this.schemaType) + " | null" : _this.schemaType + " | null";
      }

      return _this;
    };

    this.getEnumName = function (propertyName, parentKey) {
      if (parentKey === void 0) {
        parentKey = "";
      }

      return "" + toCapitalCase(parentKey) + toCapitalCase(propertyName) + ENUM_SUFFIX;
    };

    this.resolveRef = function ($ref, type) {
      if (!$ref) {
        return "";
      }

      var refType = /*#__PURE__*/addPrefixForInterface( /*#__PURE__*/toCapitalCase( /*#__PURE__*/_this.pickTypeByRef($ref)));
      return type === "array" ? refType + "[]" : refType;
    };

    this.getBasicType = function (basicType, advancedType) {
      if (basicType === void 0) {
        basicType = "";
      }

      switch (basicType) {
        case "integer":
          return "number";

        case "array":
          return _this.getTypeForArray(advancedType);

        case "":
          return advancedType || "";

        default:
          return basicType;
      }
    };

    this.getTypeForArray = function (advancedType) {
      return advancedType ? advancedType + "[]" : "Array<any>";
    };

    this.pickTypeByRef = function (str) {
      if (!str) {
        return;
      }

      var list = /*#__PURE__*/str.split("/");
      return list[list.length - 1];
    };

    this.resolveItems = function (items, type, key, parentKey) {
      if (!items) {
        return {};
      }

      var child = /*#__PURE__*/lodashEs.get(items, "items");

      if (type === "array") {
        if (child) {
          return _this.resolveItems(child, items.type, key, parentKey) + "[]";
        }

        if (!lodashEs.get(items, "$ref")) {
          return lodashEs.get(items, "type") + "[]";
        }
      }

      if (isArray(items)) {
        return lodashEs.map(items, function (item) {
          return SchemaResolver.of({
            results: _this.inputs.results,
            schema: item,
            key: key,
            parentKey: parentKey
          }).resolve().getSchemaType();
        });
      }

      return SchemaResolver.of({
        results: _this.inputs.results,
        schema: items,
        key: key,
        parentKey: parentKey
      }).resolve(type).getSchemaType();
    };

    this.resolveProperties = function (properties, required, parentKey) {
      if (properties === void 0) {
        properties = {};
      }

      if (required === void 0) {
        required = [];
      }

      return lodashEs.reduce(properties, function (o, v, k) {
        var _a;

        return tslib.__assign( /*#__PURE__*/tslib.__assign({}, o), (_a = {}, _a["" + k + (lodashEs.indexOf(required, k) > -1 ? "" : "?")] = /*#__PURE__*/SchemaResolver.of({
          results: _this.inputs.results,
          schema: v,
          key: k,
          parentKey: parentKey
        }).resolve().getSchemaType(), _a));
      }, {});
    };
  }

  SchemaResolver.of = function (inputs) {
    return new SchemaResolver(inputs);
  };

  return SchemaResolver;
}();

var generateEnums = function (definitions, key) {
  if (lodashEs.isEmpty(definitions)) {
    return "";
  }

  var enums = definitions[key];
  var hasNumber = /*#__PURE__*/lodashEs.some(enums, function (enumValue) {
    return isNumber(enumValue);
  });
  var enumName = /*#__PURE__*/lodashEs.replace(key, ENUM_SUFFIX, "");
  return hasNumber ? "export type " + enumName + " = " + enums.map(function (item) {
    return JSON.stringify(item);
  }).join("|") : "export enum " + enumName + " " + JSON.stringify( /*#__PURE__*/arrayToObject(enums)).replace(/:/gi, "=");
};

var DefinitionsResolver = function () {
  function DefinitionsResolver(components) {
    var _this = this;

    this.components = components;

    this.scanDefinitions = function () {
      var results = {};
      var requestBodies = /*#__PURE__*/lodashEs.get(_this.components, "requestBodies");
      var schemas = /*#__PURE__*/lodashEs.get(_this.components, "schemas");
      lodashEs.forEach(requestBodies, function (requestBody, requestBodyName) {
        if (isRequestBody(requestBody)) {
          return results[requestBodyName] = /*#__PURE__*/SchemaResolver.of({
            results: results,
            schema: /*#__PURE__*/lodashEs.get(requestBody, "content.application/json.schema"),
            key: requestBodyName,
            parentKey: requestBodyName
          }).resolve().getSchemaType();
        }

        return results[requestBodyName] = /*#__PURE__*/SchemaResolver.of({
          results: results,
          schema: requestBody,
          key: requestBodyName,
          parentKey: requestBodyName
        }).resolve().getSchemaType();
      });
      lodashEs.forEach(schemas, function (schema, schemaName) {
        return results[schemaName] = /*#__PURE__*/SchemaResolver.of({
          results: results,
          schema: schema,
          key: schemaName,
          parentKey: schemaName
        }).resolve().getSchemaType();
      });
      _this.resolvedDefinitions = results;
      return _this;
    };

    this.toDeclarations = function () {
      var arr = /*#__PURE__*/Object.keys(_this.resolvedDefinitions).sort().map(function (key) {
        if (lodashEs.includes(key, ENUM_SUFFIX)) {
          return generateEnums(_this.resolvedDefinitions, key);
        }

        if (_this.resolvedDefinitions[key] === "object" || lodashEs.isEmpty(_this.resolvedDefinitions[key])) {
          return "export interface " + addPrefixForInterface( /*#__PURE__*/toCapitalCase(key)) + " {[key:string]:any}";
        }

        var val = /*#__PURE__*/toTypes(_this.resolvedDefinitions[key]);

        if (val) {
          return "export interface " + addPrefixForInterface( /*#__PURE__*/toCapitalCase(key)) + " " + val;
        }
      });
      return lodashEs.compact(arr);
    };
  }

  DefinitionsResolver.of = function (components) {
    return new DefinitionsResolver(components);
  };

  return DefinitionsResolver;
}();

var PathResolver = function () {
  function PathResolver(paths) {
    var _this = this;

    this.paths = paths;
    this.resolvedPaths = [];
    this.extraDefinitions = {};
    this.contentType = {};

    this.resolve = function () {
      _this.resolvedPaths = /*#__PURE__*/lodashEs.reduce(_this.paths, function (results, path, pathName) {
        return tslib.__spreadArray( /*#__PURE__*/tslib.__spreadArray([], results, true), /*#__PURE__*/_this.resolvePath(path, pathName), true);
      }, []);
      return _this;
    };

    this.getRequestURL = function (pathName) {
      return pathName.split(SLASH).map(function (p) {
        return _this.isPathParam(p) ? "$" + p : p;
      }).join(SLASH);
    };

    this.isPathParam = function (str) {
      return str.startsWith("{");
    };

    this.resolveOperation = function (operation) {
      var pickParamsByType = /*#__PURE__*/_this.pickParams(operation.parameters);

      var headerParams = /*#__PURE__*/pickParamsByType("header");
      var params = {
        pathParams: /*#__PURE__*/pickParamsByType("path"),
        queryParams: /*#__PURE__*/pickParamsByType("query"),
        bodyParams: /*#__PURE__*/pickParamsByType("body"),
        formDataParams: /*#__PURE__*/pickParamsByType("cookie")
      };
      return tslib.__assign( /*#__PURE__*/tslib.__assign({
        operationId: operation.operationId,
        TResp: /*#__PURE__*/_this.getResponseTypes(operation.responses),
        TReq: /*#__PURE__*/_this.getRequestTypes(params, operation.operationId, /*#__PURE__*/lodashEs.get(operation, "requestBody")),
        THeader: /*#__PURE__*/_this.getPathParamsTypes(headerParams)
      }, /*#__PURE__*/_this.getParamsNames(params)), /*#__PURE__*/_this.getRequestBodyName( /*#__PURE__*/lodashEs.get(operation, "requestBody"), operation.operationId));
    };

    this.getParamsNames = function (params) {
      var getNames = function (list) {
        return lodashEs.isEmpty(list) ? [] : lodashEs.map(list, function (item) {
          return item.name;
        });
      };

      return {
        pathParams: /*#__PURE__*/getNames(params.pathParams),
        queryParams: /*#__PURE__*/getNames(params.queryParams),
        bodyParams: /*#__PURE__*/getNames(params.bodyParams),
        formDataParams: /*#__PURE__*/getNames(params.formDataParams)
      };
    };

    this.getRequestTypes = function (params, operationId, requestBody) {
      return tslib.__assign( /*#__PURE__*/tslib.__assign( /*#__PURE__*/tslib.__assign( /*#__PURE__*/tslib.__assign( /*#__PURE__*/tslib.__assign({}, /*#__PURE__*/_this.getPathParamsTypes(params.pathParams)), /*#__PURE__*/_this.getBodyAndQueryParamsTypes(params.bodyParams)), /*#__PURE__*/_this.getBodyAndQueryParamsTypes(params.queryParams)), /*#__PURE__*/_this.getFormDataParamsTypes(params.formDataParams)), /*#__PURE__*/_this.getRequestBodyTypes(operationId, requestBody));
    };

    this.getPathParamsTypes = function (pathParams) {
      return pathParams.reduce(function (results, param) {
        var _a;

        var schema = /*#__PURE__*/lodashEs.get(param, "schema");

        if (isSchema(schema)) {
          return tslib.__assign( /*#__PURE__*/tslib.__assign({}, results), (_a = {}, _a["" + param.name + (param.required ? "" : "?")] = schema.type === "integer" ? "number" : schema.type, _a));
        }

        return tslib.__assign({}, results);
      }, {});
    };

    this.getBodyAndQueryParamsTypes = function (bodyParams) {
      return bodyParams.reduce(function (results, param) {
        var _a;

        return tslib.__assign( /*#__PURE__*/tslib.__assign({}, results), (_a = {}, _a["" + param.name + (param.required ? "" : "?")] = /*#__PURE__*/SchemaResolver.of({
          results: _this.extraDefinitions,
          schema: param.schema,
          key: param.name,
          parentKey: param.name
        }).resolve().getSchemaType(), _a));
      }, {});
    };

    this.getFormDataParamsTypes = function (formDataParams) {
      return formDataParams.reduce(function (results, param) {
        var _a, _b;

        if (param.schema) {
          return tslib.__assign( /*#__PURE__*/tslib.__assign({}, results), (_a = {}, _a["" + param.name + (param.required ? "" : "?")] = /*#__PURE__*/SchemaResolver.of({
            results: _this.extraDefinitions,
            schema: param.schema,
            key: param.name,
            parentKey: param.name
          }).resolve().getSchemaType(), _a));
        }

        return tslib.__assign( /*#__PURE__*/tslib.__assign({}, results), (_b = {}, _b["" + param.name + (param.required ? "" : "?")] = param.type === "file" ? "File" : param.type, _b));
      }, {});
    }; // TODO: handle Response or Reference


    this.getResponseTypes = function (responses) {
      return SchemaResolver.of({
        results: _this.extraDefinitions,
        // TODO: handle other content type here
        schema: lodashEs.get(responses, "200.content.application/json.schema") || lodashEs.get(responses, "200.content.*/*.schema") || lodashEs.get(responses, "201.content.application/json.schema") || lodashEs.get(responses, "201.content.*/*.schema")
      }).resolve().getSchemaType();
    }; // TODO: when parameters has enum


    this.pickParams = function (parameters) {
      return function (type) {
        return lodashEs.filter(parameters, function (param) {
          return param.in === type;
        });
      };
    };
  }

  PathResolver.of = function (paths) {
    return new PathResolver(paths);
  };

  PathResolver.prototype.resolvePath = function (path, pathName) {
    var _this = this;

    var operations = /*#__PURE__*/lodashEs.pick(path, HTTP_METHODS);
    return Object.keys(operations).map(function (httpMethod) {
      return tslib.__assign({
        url: /*#__PURE__*/_this.getRequestURL(pathName),
        method: httpMethod
      }, /*#__PURE__*/_this.resolveOperation(operations[httpMethod]));
    });
  };

  PathResolver.prototype.getContentType = function (operationId, key) {
    var _a; // in openAPI spec, the key of content in requestBody field is content type


    lodashEs.assign(this.contentType, (_a = {}, _a[operationId] = key, _a));
  };

  PathResolver.prototype.getRequestBodyTypes = function (operationId, requestBody) {
    var _a;

    var _this = this;

    if (isRequestBody(requestBody)) {
      return lodashEs.reduce( /*#__PURE__*/lodashEs.get(requestBody, "content"), function (results, content, key) {
        var _a;

        _this.getContentType(operationId, key);

        return tslib.__assign( /*#__PURE__*/tslib.__assign({}, results), (_a = {}, _a[operationId + "Request"] = /*#__PURE__*/SchemaResolver.of({
          results: _this.extraDefinitions,
          schema: content.schema,
          key: operationId + "Request",
          parentKey: operationId + "Request"
        }).resolve().getSchemaType(), _a));
      }, {});
    }

    return _a = {}, _a[operationId + "Request"] = /*#__PURE__*/SchemaResolver.of({
      results: this.extraDefinitions,
      schema: requestBody,
      key: operationId + "Request",
      parentKey: operationId + "Request"
    }).resolve().getSchemaType(), _a;
  };

  PathResolver.prototype.getRequestBodyName = function (requestBody, operationId) {
    if (requestBody) {
      return {
        requestBody: operationId + "Request"
      };
    }
  };

  return PathResolver;
}();

exports.DefinitionsResolver = DefinitionsResolver;
exports.PathResolver = PathResolver;
exports.SchemaResolver = SchemaResolver;
